@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2003 Fons Adriaensen <fons.adriaensen@skynet.be>
+    Copyright (C) 2003-2008 Fons Adriaensen <fons@kokkinizita.net>
     
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,10 +20,34 @@
 #include "lfqueue.h"
 
 
+Lfq_u8::Lfq_u8 (int size) : _size (size), _mask (_size - 1), _nwr (0), _nrd (0)
+{
+    assert (!(_size & _mask));
+    _data = new uint8_t [_size];
+}
+
+Lfq_u8::~Lfq_u8 (void)
+{
+    delete[] _data;
+} 
+
+
+Lfq_u16::Lfq_u16 (int size) : _size (size), _mask (_size - 1), _nwr (0), _nrd (0)
+{
+    assert (!(_size & _mask));
+    _data = new uint16_t [_size];
+}
+
+Lfq_u16::~Lfq_u16 (void)
+{
+    delete[] _data;
+} 
+
+
 Lfq_u32::Lfq_u32 (int size) : _size (size), _mask (_size - 1), _nwr (0), _nrd (0)
 {
     assert (!(_size & _mask));
-    _data = new U32 [_size];
+    _data = new uint32_t [_size];
 }
 
 Lfq_u32::~Lfq_u32 (void)
