@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2005 Fons Adriaensen <fons.adriaensen@skynet.be>
+    Copyright (C) 2003-2008 Fons Adriaensen <fons@kokkinizita.net>
     
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -27,7 +27,10 @@
 #include "global.h"
 
 
-Divis::Divis (void) : _flags (0), _dmask (0), _nrank (0)
+Divis::Divis (void) :
+    _flags (0),
+    _dmask (0),
+    _nrank (0)
 {
     *_label = 0;
     _param [SWELL]._val = SWELL_DEF;
@@ -43,13 +46,59 @@ Divis::Divis (void) : _flags (0), _dmask (0), _nrank (0)
 
 
 
-Model::Model (Lfq_u32 *commq, const char *appid, const char *stops, const char *instr, const char *waves, bool uhome) :
-    A_thread ("Model"), _commq (commq), _appid (appid), _stops (stops), _uhome (uhome), _ready (false), 
-    _nasect (0), _ndivis (0), _nkeybd (0), _ngroup (0), _count (0), _bank (0), _pres (0), _audio (0), _midi (0)
+Keybd::Keybd (void) :
+    _flags (0)
+{
+    *_label = 0;
+}
+
+
+Ifelm::Ifelm (void) :
+    _state (0)
+{
+    *_label = 0;
+    *_mnemo = 0;
+}
+
+
+Group::Group (void) :
+    _nifelm (0)
+{
+    *_label = 0;
+}
+
+
+Model::Model (Lfq_u32      *qcomm,
+              Lfq_u8       *qmidi,
+	      uint16_t     *midimap,
+              const char   *appname,
+              const char   *stops,
+              const char   *instr,
+              const char   *waves,
+              bool          uhome) :
+    A_thread ("Model"),
+    _qcomm (qcomm),
+    _qmidi (qmidi), 
+    _midimap (midimap),
+    _appname (appname),
+    _stops (stops),
+    _uhome (uhome),
+    _ready (false), 
+    _nasect (0),
+    _ndivis (0),
+    _nkeybd (0),
+    _ngroup (0),
+    _count (0),
+    _bank (0),
+    _pres (0),
+    _sc_cmode (0),
+    _sc_group (0),
+    _audio (0),
+    _midi (0)
 {
     sprintf (_instr, "%s/%s", stops, instr);
     sprintf (_waves, "%s/%s", stops, waves);
-    memset (_chbits, 0, 16 * sizeof (U16));
+    memset (_midimap, 0, 16 * sizeof (uint16_t));
     memset (_preset, 0, NBANK * NPRES * sizeof (Preset *));
 }
 
@@ -66,7 +115,9 @@ void Model::thr_main (void)
     int E;
 
     init ();     
-    while ((E = get_event ()) != EV_EXIT)
+    set_time (0);
+    inc_time (100000);
+    while ((E = get_event_timed ()) != EV_EXIT)
     {
 	switch (E)
 	{
@@ -77,6 +128,15 @@ void Model::thr_main (void)
             proc_mesg (get_message ());
 	    break;
 
+	case EV_TIME:    
+	    inc_time (50000);
+	    proc_qmidi ();
+	    break;
+
+	case EV_QMIDI:
+	    proc_qmidi ();
+	    break;
+
 	default:
 	    ;
 	}
@@ -101,63 +161,78 @@ void Model::fini (void)
 
 void Model::proc_mesg (ITC_mesg *M)
 {
+    // Handle commands from other threads, including
+    // the user interface.
+
     switch (M->type ())
     {
     case MT_IFC_ELCLR:
     case MT_IFC_ELSET:
     case MT_IFC_ELXOR:
     {
+	// Set, reset or toggle a stop.
         M_ifc_ifelm *X = (M_ifc_ifelm *) M;
         set_ifelm (X->_group, X->_ifelm, X->type () - MT_IFC_ELCLR);
 	break;
     }
+    case MT_IFC_ELATT:
+    {
+	send_event (TO_IFACE, M);
+	M = 0;
+	break;
+    }
     case MT_IFC_GRCLR:
     {
+	// Reset a group of stops.
         M_ifc_ifelm *X = (M_ifc_ifelm *) M;
         clr_group (X->_group);
 	break;
     }
     case MT_IFC_AUPAR:
     {
+	// Set audio section parameter.
         M_ifc_aupar *X = (M_ifc_aupar *) M;
         set_aupar (X->_srcid, X->_asect, X->_parid, X->_value);
 	break;
     }
     case MT_IFC_DIPAR:
     {
+	// Set division parameter.
         M_ifc_dipar *X = (M_ifc_dipar *) M;
         set_dipar (X->_srcid, X->_divis, X->_parid, X->_value);
 	break;
     }
     case MT_IFC_RETUNE:
     {
+	// Recalculate all wavetables.
         M_ifc_retune *X = (M_ifc_retune *) M;
         retune (X->_freq, X->_temp);
 	break;
     }
-    case MT_IFC_ASOFF:
+    case MT_IFC_ANOFF:
     {
-	M_ifc_asoff *X = (M_ifc_asoff *) M;
+	// All notes off.
+	M_ifc_anoff *X = (M_ifc_anoff *) M;
         midi_off (X->_bits);
 	break;
     }
     case MT_IFC_MCSET:
     {
+	// Store midi routing preset.
         int index;
-
 	M_ifc_chconf *X = (M_ifc_chconf *) M;
         index = X->_index;
         if (index >= 0)
 	{ 
 	    if (index >= 8) break;
-            memcpy (_chconf [X->_index]._bits, X->_bits, 16 * sizeof (U16));
+            memcpy (_chconf [X->_index]._bits, X->_bits, 16 * sizeof (uint16_t));
 	}  
         set_mconf (X->_index, X->_bits);
     }
     case MT_IFC_MCGET:
     {
+	// Recall midi routing preset.
         int index;
-
 	M_ifc_chconf *X = (M_ifc_chconf *) M;
         index = X->_index;
         if (index >= 0)
@@ -169,6 +244,7 @@ void Model::proc_mesg (ITC_mesg *M)
     }
     case MT_IFC_PRRCL:
     {
+	// Load a preset.
 	M_ifc_preset *X = (M_ifc_preset *) M;
         if (X->_bank < 0) X->_bank = _bank;
         set_state (X->_bank, X->_pres);       
@@ -176,38 +252,44 @@ void Model::proc_mesg (ITC_mesg *M)
     }
     case MT_IFC_PRDEC:
     {
+	// Decrement preset and load.
 	if (_pres > 0) set_state (_bank, --_pres);
         break;
     }
     case MT_IFC_PRINC:
     {
+	// Increment preset and load.
 	if (_pres < NPRES - 1) set_state (_bank, ++_pres);
         break;
     }
     case MT_IFC_PRSTO:
     {
-	M_ifc_preset *X = (M_ifc_preset *) M;
-        U32           d [NGROUP];
+	// Store a preset.
+	M_ifc_preset  *X = (M_ifc_preset *) M;
+        uint32_t       d [NGROUP];
         get_state (d);
         set_preset (X->_bank, X->_pres, d);         
         break;
     }
     case MT_IFC_PRINS:
     {
+	// Insert a preset.
 	M_ifc_preset *X = (M_ifc_preset *) M;
-        U32           d [NGROUP];
+        uint32_t     d [NGROUP];
         get_state (d);
         ins_preset (X->_bank, X->_pres, d);         
         break;
     }
     case MT_IFC_PRDEL:
     {
+	// Delete a preset.
 	M_ifc_preset *X = (M_ifc_preset *) M;
         del_preset (X->_bank, X->_pres);         
         break;
     }
     case MT_IFC_PRGET:
     {
+	// Read a preset.
 	M_ifc_preset *X = (M_ifc_preset *) M;
         X->_stat = get_preset (X->_bank, X->_pres, X->_bits);
         send_event (TO_IFACE, M);
@@ -216,6 +298,7 @@ void Model::proc_mesg (ITC_mesg *M)
     }
     case MT_IFC_EDIT:
     {
+	// Start editing a stop.
 	M_ifc_edit *X = (M_ifc_edit *) M;
         Rank       *R = find_rank (X->_group, X->_ifelm); 
         if (_ready && R)
@@ -228,22 +311,26 @@ void Model::proc_mesg (ITC_mesg *M)
     }
     case MT_IFC_APPLY:
     {
+	// Apply edited stop.
 	M_ifc_edit *X = (M_ifc_edit *) M;
         if (_ready) recalc (X->_group, X->_ifelm);
 	break;
     }
     case MT_IFC_SAVE:
+	// Save presets, midi presets, and wavetables.
 	save ();
         break;
 
     case MT_LOAD_RANK:
     case MT_CALC_RANK:
     {
+	// Load a rank into a division.
         M_def_rank *X = (M_def_rank *) M; 
         _divis [X->_divis]._ranks [X->_rank]._wave = X->_wave;
 	break;
     }
     case MT_AUDIO_INFO:
+	// Initialisation info from audio thread.
         _audio = (M_audio_info *) M; 
         M = 0;
         if (_midi)
@@ -255,6 +342,7 @@ void Model::proc_mesg (ITC_mesg *M)
 	break;
 
     case MT_MIDI_INFO:
+	// Initialisation info from midi thread.
         _midi = (M_midi_info *) M; 
         M = 0;
         if (_audio)
@@ -266,6 +354,7 @@ void Model::proc_mesg (ITC_mesg *M)
 	break;
 
     case MT_AUDIO_SYNC:
+	// Wavetable calculation done.
         send_event (TO_IFACE, new ITC_mesg (MT_IFC_READY));
         _ready = true;
 	break;
@@ -277,6 +366,82 @@ void Model::proc_mesg (ITC_mesg *M)
 }    
 
 
+void Model::proc_qmidi (void)
+{
+    int c, d, p, t, v;
+
+    // Handle commands from the qmidi queue. These are coming
+    // from either the midi thread (ALSA), or the audio thread
+    // (JACK). They are encoded as raw MIDI, except that all
+    // messages are 3 bytes. All command have already been
+    // checked at the sending side.
+
+    while (_qmidi->read_avail () >= 3)
+    {
+	t = _qmidi->read (0);
+	p = _qmidi->read (1);
+	v = _qmidi->read (2);
+	_qmidi->read_commit (3);
+	c = t & 0x0F;
+        d = (_midimap [c] >>  8) & 7;
+	switch (t & 0xF0)
+        {
+	case 0x90:
+	    // Program change from notes 24..33.
+	    set_state (_bank, p - 24);
+	    break;
+
+	case 0xB0:
+	    // Controllers.
+            switch (p)
+	    {
+	    case MIDICTL_SWELL:
+		// Swell pedal
+                set_dipar (SRC_MIDI_PAR, d, 0, SWELL_MIN + v * (SWELL_MAX - SWELL_MIN) / 127.0f);
+                break;
+
+	    case MIDICTL_TFREQ:
+		// Tremulant frequency
+                set_dipar (SRC_MIDI_PAR, d, 1, TFREQ_MIN + v * (TFREQ_MAX - TFREQ_MIN) / 127.0f);
+                break;
+
+	    case MIDICTL_TMODD:
+		// Tremulant amplitude
+                set_dipar (SRC_MIDI_PAR, d, 2, TMODD_MIN + v * (TMODD_MAX - TMODD_MIN) / 127.0f);
+                break;
+
+            case MIDICTL_BANK:
+                // Preset bank.
+ 	        if (v < NBANK) _bank = v;
+                break;
+
+	    case MIDICTL_IFELM:
+		// Stop control.
+                if (v & 64)
+  	        {
+		    // Set mode or clear group.
+                    _sc_cmode = (v >> 4) & 3;
+                    _sc_group = v & 7; 
+                    if (_sc_cmode == 0) clr_group (_sc_group);
+		}
+                else if (_sc_cmode)
+		{
+		    // Set, reset or toggle stop.
+                    set_ifelm (_sc_group, v & 31, _sc_cmode - 1);
+		}         
+                break;
+	    }
+	    break;
+
+	case 0xC0:
+	    // Program change.
+	    if (p < NPRES) set_state (_bank, p);
+	    break;
+	}
+    }
+}
+
+
 void Model::init_audio (void)
 {
     int          d;
@@ -309,7 +474,7 @@ void Model::init_iface (void)
     M->_stops  = _stops;
     M->_waves  = _waves;
     M->_instr  = _instr;
-    M->_appid  = _appid;
+    M->_appid  = _appname;
     M->_client = _midi->_client;
     M->_ipport = _midi->_ipport;
     M->_nasect = _nasect;
@@ -317,20 +482,20 @@ void Model::init_iface (void)
     M->_ndivis = _ndivis;
     M->_ngroup = _ngroup;
     M->_ntempe = NSCALES;
-    for (i = 0; i < _nkeybd; i++)
+    for (i = 0; i < NKEYBD; i++)
     {
         K = _keybd + i;
 	M->_keybdd [i]._label = K->_label;
 	M->_keybdd [i]._flags = K->_flags;
     }
-    for (i = 0; i < _ndivis; i++)
+    for (i = 0; i < NDIVIS; i++)
     {
         D = _divis + i;
 	M->_divisd [i]._label = D->_label;
 	M->_divisd [i]._flags = D->_flags;
 	M->_divisd [i]._asect = D->_asect;
     }
-    for (i = 0; i < _ngroup; i++)
+    for (i = 0; i < NGROUP; i++)
     {
         G = _group + i;
 	M->_groupd [i]._label  = G->_label;
@@ -449,10 +614,10 @@ void Model::set_ifelm (int g, int i, int m)
     if (I->_state != s)
     {
 	I->_state = s;
-        if (_commq->wr_avail ())
+        if (_qcomm->write_avail ())
 	{
-	    *(_commq->wr_ptr ()) = s ? I->_action1 : I->_action0;  
-            _commq->wr_commit ();
+	    _qcomm->write (0, s ? I->_action1 : I->_action0);  
+            _qcomm->write_commit (1);
             send_event (TO_IFACE, new M_ifc_ifelm (MT_IFC_ELCLR + s, g, i));         
 	}
     }
@@ -474,10 +639,10 @@ void Model::clr_group (int g)
         if (I->_state)
         {
 	    I->_state = 0;
-            if (_commq->wr_avail ())
+            if (_qcomm->write_avail ())
 	    {
-	        *(_commq->wr_ptr ()) = I->_action0;  
-               _commq->wr_commit ();
+	       _qcomm->write (0, I->_action0);  
+               _qcomm->write_commit (1);
 	    }
 	}
     }
@@ -485,10 +650,10 @@ void Model::clr_group (int g)
 }
 
 
-void Model::get_state (U32 *d)
+void Model::get_state (uint32_t *d)
 {
     int    g, i;
-    U32    s;
+    uint32_t    s;
     Group  *G;
     Ifelm  *I;
 
@@ -509,7 +674,7 @@ void Model::get_state (U32 *d)
 void Model::set_state (int bank, int pres)
 {
     int    g, i;
-    U32    d [NGROUP], s;
+    uint32_t    d [NGROUP], s;
     Group  *G;
 
     _bank = bank;
@@ -547,23 +712,24 @@ void Model::set_aupar (int s, int a, int p, float v)
 void Model::set_dipar (int s, int d, int p, float v)
 {
     Fparm  *P;
+    union { uint32_t i; float f; } u;
     
     P = _divis [d]._param + p;
     if (v < P->_min) v = P->_min;
     if (v > P->_max) v = P->_max;
     P->_val = v;
-    if (_commq->wr_avail () >= 2)
+    if (_qcomm->write_avail () >= 2)
     {
-	*(_commq->wr_ptr ()) = (17 << 24) | (d << 16) | (p << 8);
-        _commq->wr_commit ();
-	*((float *)(_commq->wr_ptr ())) = v;
-        _commq->wr_commit ();
+	u.f = v;
+	_qcomm->write (0, (17 << 24) | (d << 16) | (p << 8));
+	_qcomm->write (1, u.i);
+        _qcomm->write_commit (2);
         send_event (TO_IFACE, new M_ifc_dipar (s, d, p, v));         
     }
 }
 
 
-void Model::set_mconf (int i, U16 *d)
+void Model::set_mconf (int i, uint16_t *d)
 {
     int j, a, b;
     
@@ -573,7 +739,7 @@ void Model::set_mconf (int i, U16 *d)
         a = d [j];
 	b =  (a & 0x1000) ? (_keybd [a & 7]._flags & 127) : 0;
         b |= a & 0x7700;
-        _chbits [j] = b;
+        _midimap [j] = b;
     }
     send_event (TO_IFACE, new M_ifc_chconf (MT_IFC_MCSET, i, d));         
 }
@@ -582,10 +748,10 @@ void Model::set_mconf (int i, U16 *d)
 void Model::midi_off (int mask)
 {
     mask &= 127;
-    if (_commq->wr_avail ())
+    if (_qcomm->write_avail ())
     {
-        *(_commq->wr_ptr ()) = (2 << 24) | (mask << 16) | mask;
-        _commq->wr_commit ();
+        _qcomm->write (0, (2 << 24) | (mask << 16) | mask);
+        _qcomm->write_commit (1);
     }
 }
 
@@ -733,7 +899,7 @@ int Model::read_instr (void)
 		    K = _keybd + k; 
                     strcpy (K->_label, t1);
                     K->_flags = 1 << k;
-                    if (p [1] == 'p') K->_flags |= 64 | Keybd::IS_PEDAL;
+                    if (p [1] == 'p') K->_flags |= HOLD_MASK | Keybd::IS_PEDAL;
 		}
 	    }
 	}
@@ -1086,7 +1252,7 @@ int Model::write_instr (void)
 }
 
 
-int Model::get_preset (int bank, int pres, U32 *bits)
+int Model::get_preset (int bank, int pres, uint32_t *bits)
 {
     int     k;
     Preset  *P;
@@ -1102,7 +1268,7 @@ int Model::get_preset (int bank, int pres, U32 *bits)
 } 
 
 
-void Model::set_preset (int bank, int pres, U32 *bits)
+void Model::set_preset (int bank, int pres, uint32_t *bits)
 {
     int     k;
     Preset  *P;
@@ -1118,7 +1284,7 @@ void Model::set_preset (int bank, int pres, U32 *bits)
 }
 
 
-void Model::ins_preset (int bank, int pres, U32 *bits)
+void Model::ins_preset (int bank, int pres, uint32_t *bits)
 {
     int     j, k;
     Preset  *P;
@@ -1154,17 +1320,24 @@ int Model::read_presets (void)
     FILE           *F;
     Preset         *P;
 
+    F = 0;
     if (_uhome)
     {
         p = (unsigned char *)(getenv ("HOME"));
         if (p) sprintf (name, "%s/.aeolus-presets", p);
         else strcpy (name, ".aeolus-presets");
+        F = fopen (name, "r"); 
+	if (F == 0)
+        {
+	    fprintf (stderr, "Can't open '%s' for reading\n", name);
+        }
     }
-    else
+    if (F == 0)
     {
 	sprintf (name, "%s/presets", _instr);
+        F = fopen (name, "r"); 
     }
-    if (! (F = fopen (name, "r"))) 
+    if (F == 0)
     {
 	fprintf (stderr, "Can't open '%s' for reading\n", name);
         return 1;
